/*
 * Copyright (c) 2018 Nordic Semiconductor ASA
 *
 * SPDX-License-Identifier: LicenseRef-BSD-5-Clause-Nordic
 */

#include <gpio.h>
#include <net/socket.h>
#include <nrf9160.h>
#include <stdio.h>
#include <string.h>
#include <uart.h>
#include <zephyr.h>

struct device *gpio_dev;
static const u8_t led_pins[] = { LED0_GPIO_PIN, LED1_GPIO_PIN, LED2_GPIO_PIN,
				 LED3_GPIO_PIN };

ISR_DIRECT_DECLARE(timer_handler)
{
	static bool toggle_in_isr;
	if (NRF_TIMER1_NS->EVENTS_COMPARE[0])
	{
		NRF_TIMER1_NS->EVENTS_COMPARE[0] = 0;
		toggle_in_isr = !toggle_in_isr;
		(void)gpio_pin_write(gpio_dev, led_pins[1], toggle_in_isr);
	}
	ISR_DIRECT_PM();
	return 1;
}

void timer_init(void)
{
	IRQ_DIRECT_CONNECT(TIMER1_IRQn, 7, timer_handler, 0);
	irq_enable(TIMER1_IRQn);
	NRF_TIMER1_NS->PRESCALER = 8;	
	NRF_TIMER1_NS->CC[0] = 50000;
	NRF_TIMER1_NS->SHORTS = TIMER_SHORTS_COMPARE0_CLEAR_Enabled << TIMER_SHORTS_COMPARE0_CLEAR_Pos;
	NRF_TIMER1_NS->INTENSET = TIMER_INTENSET_COMPARE0_Enabled << TIMER_INTENSET_COMPARE0_Pos;
	NRF_TIMER1_NS->TASKS_START = 1;

}

int dk_leds_init(void)
{
	int err = 0;

	gpio_dev = device_get_binding(DT_GPIO_P0_DEV_NAME);
	if (!gpio_dev) {
		printk("Cannot bind gpio device");
		return -ENODEV;
	}

	for (size_t i = 0; i < ARRAY_SIZE(led_pins); i++) {
		err = gpio_pin_configure(gpio_dev, led_pins[i], GPIO_DIR_OUT);

		if (err) {
			printk("Cannot configure LED gpio");
			return err;
		}
	}
	return err;
}

int main(void)
{
	bool toggle = false;
	printk("timer demo\n\r");
	int err = dk_leds_init();
	timer_init();

	if (err) {
		printk("Error gpio config: %d", err);
		return;
	}
	while (1) {
		/* Using native gpio library adds delay */
		(void)gpio_pin_write(gpio_dev, led_pins[0], toggle);
		toggle = !toggle;
		k_sleep(100);
	}
}
